/* $Workfile: Dsp55Demo1.c $   $Revision: 7 $    $Modtime: 11/17/03 4:58p $  */
/*--------------------------------------------------------------------------*\

  Odin TeleSystems Inc.

  SAA-1007-1 OTX DSP C54x Software Development Kit (SDK)

  Demo1

  This demo provides a skeleton for a OTX DSP Application.

  It also allows the host application to control the blinking of a
  heart-beat LED.

  Use together with host application Dsp55Demo1.c

\*--------------------------------------------------------------------------*/

#include <string.h>   // for memset()     

#include "..\Lib\Dsp55Apicfg.h" // automatically generated by the gconfgen tool
#include <csl.h>
#include <csl_dma.h>
#include <csl_mcbsp.h>
#include <csl_irq.h>
#include <csl_chip.h>
#include <csl_timer.h>

#include <Dsp55Api.h>
#include <Dsp55Ioctl.h>

#include "Dsp55Demo1Io.h"  

volatile unsigned short g_nEntryPoint = 0x00;

/*--------------------------------------------------------------------------*\
  Private data
\*--------------------------------------------------------------------------*/
static volatile short g_blinkEnable     = OTXDSP_TRUE;
static volatile long  g_blinkCount      = 0;
static volatile long  g_blinkCountMax   = 20000L;
static volatile short g_blinkXfVal      = OTXDSP_FALSE;

unsigned short g_hHandle = OTXDSP_INVALID_HANDLE;  // Handle passed to us from the host user application

Demo1NotifyGetDebugDataS g_debugDataS;
static unsigned short g_coDebugDataNotifications = 1;

/*--------------------------------------------------------------------------*\
  Function Prototypes
\*--------------------------------------------------------------------------*/
//static void ProcessRxSamples(void);
//static void GenerateTxSamples(void);


/*--------------------------------------------------------------------------*\
  @func
  
    main() is called from _c_int00 which is the entry point for the program.
    _c_int00 will first initialize all global data before running main().
    
    The function CSL_cfgInit() will also be called before main() is called.
    
    This function will initialize global data and peripherals and then terminate
    to allow the DSP/BIOS tasks to run.
    
\*--------------------------------------------------------------------------*/
main() {

  LOG_printf(&trace, "Demo program started\n");  

  // Run Standard OTX DSP SDK API Initializations
  OtxDspInit();
  
  // Initialize application specific data
  memset(&g_debugDataS, 0, sizeof(Demo1NotifyGetDebugDataS));

  LOG_printf(&trace, "main() terminating... DSP BIOS tasks should be running.\n");  

}

/*--------------------------------------------------------------------------*\
  @func

    This function is the main process task thread. It will go to sleep
    until a semaphore is posted from an ISR, which will wake up the thread
    to allow it to execute and process data (highway receive data, 
    highway transmit data, and control data from the host).
    
\*--------------------------------------------------------------------------*/
Void TskDataProcessing(Arg value_arg)
{
  Int value = ArgToInt(value_arg);
  Bool semRc = 1; 

  LOG_printf(&trace, "Data Processing task started (Arg=%d)\n", value);  

  // Loop forever
  for(;;) {

    // check if we have RX samples available, process them. E.g. run detection algorithms on them
    if (g_bGotDmaRxInterrupt) {  // g_bGotDmaRxInterrupt updated by the interrupt service routine
      g_bGotDmaRxInterrupt = OTXDSP_FALSE;
      OtxDspServiceRxSerialPort();
    }

    // check if more samples are needed to be transmitted, generate more samples.
    if (g_bGotDmaTxInterrupt) {  // g_bGotDmaTxInterrupt updated by the interrupt service routine
      g_bGotDmaTxInterrupt = OTXDSP_FALSE;
      OtxDspServiceTxSerialPort();
    }

    // check if the host has requested our attention. If so, service his request
    if (g_bGotHostPortInterrupt) {  // g_bGotHostPortInterrupt updated by the interrupt service routine
      g_bGotHostPortInterrupt = OTXDSP_FALSE;
        
      if (*g_h2dCmdAddrP != OTXDSP_SDK_IO_NO_COMMAND) { 
        // We have an IO Control command to process
        OtxDspServiceIoControl();
      }
    }


    // Sleep until we get a hardware interrupt (serial port or host port)
    semRc = SEM_pend(&semProcessing, 20); //SYS_FOREVER); 
    
    if (!semRc) {
      g_debugDataS.nData1++;
    }


  }  /* for */

}


/*--------------------------------------------------------------------------*\
  @func
    
    Idle task. 
  
    Blink the heart-beat LED  (green LED2).

\*--------------------------------------------------------------------------*/
Void IdlBlink(Arg value_arg)
{
  Int value = ArgToInt(value_arg); 

  LOG_printf(&trace, "Blink LED task started (Arg=%d)\n", value);  

  if (g_blinkCount++ >= g_blinkCountMax) {
    g_blinkCount = 0;
  
    if (g_blinkXfVal) {
      g_blinkXfVal = OTXDSP_FALSE;
      C55X_SET_LED2_ON;   
    } else {
      g_blinkXfVal = OTXDSP_TRUE;
      C55X_SET_LED2_OFF;   
    } 
        
  }

}


/*--------------------------------------------------------------------------*\
  @func

    Call-back routine called by the Interrupt Service Routine when the
    host has issued a command to the HPI and requests our attention.
  @comm
    This function is called from the processing task.
    TO BE RE-IMPLEMENTED BY THE USER.
\*--------------------------------------------------------------------------*/
void OtxDspServiceIoControl(
  void
)
{
  unsigned short nDataLen      = *((unsigned short *) (OTXDSP_SDK_IOCTL_DATA_LEN_ADDR/2));
  unsigned short hHandle       = *((unsigned short *) (OTXDSP_SDK_IOCTL_HANDLE_ADDR/2));
  unsigned short nTaskRef      = *((unsigned short *) (OTXDSP_SDK_IOCTL_TASKREF_ADDR/2));
  //short          bDataComplete = *((unsigned short *) (OTXDSP_SDK_IOCTL_CODE_ADDR/2)) & OTXDSP_SDK_MASK_IO_DATA_COMPLETE;
  //short          bDataPadded   = *((unsigned short *) (OTXDSP_SDK_IOCTL_CODE_ADDR/2)) & OTXDSP_SDK_MASK_IO_DATA_PADDED;
  unsigned short nCode         = *((unsigned short *) (OTXDSP_SDK_IOCTL_CODE_ADDR/2)) & OTXDSP_SDK_MASK_IO_CODE;


  // Standard SDK IO Control codes
  if (nCode == OTXDSP_SDK_IO_CREATE) {
    if (g_hHandle == OTXDSP_INVALID_HANDLE) {
      // The handle is not set
      g_hHandle = hHandle;
      OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_S_OK);
    } else {
      // This demo only supports one instance of the logical device. Return an error code.
      OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_E_OUT_OF_RESOURCES);
    }

  } else if (nCode == OTXDSP_SDK_IO_CLOSE) {
    g_blinkEnable = OTXDSP_FALSE;
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_S_OK);

  } else if (nCode == OTXDSP_SDK_IO_ENABLE) {
    g_blinkEnable = OTXDSP_TRUE;
    g_blinkCount = 0;
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_S_OK);

  } else if (nCode == OTXDSP_SDK_IO_DISABLE) {
    g_blinkEnable = OTXDSP_FALSE;
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_S_OK);

  } else if (nCode == OTXDSP_SDK_IO_CONNECT) {
    // This simple demo program does not support pipe connections
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_E_NOT_SUPPORTED);

  } else if (nCode == OTXDSP_SDK_IO_DISCONNECT) {
    // This simple demo program does not support pipe connections
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_E_NOT_SUPPORTED);

  } else if (nCode == OTXDSP_SDK_IO_RESET) {
    g_blinkCount = 0;
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_S_OK);

  } else if (nCode == OTXDSP_SDK_IO_INIT_BSP) {
    //OtxDspInitSerialPort(); 
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_S_OK);

  } else if (nCode == OTXDSP_SDK_IO_START_PROGRAM) {
    unsigned short dspRc;
    dspRc = OtxDspStartProgram((OtxDspSdkIoInStartProgramS *)(OTXDSP_SDK_IOCTL_DATA_ADDR/2));

    if (dspRc == OTXDSP_SDK_S_OK) {
      C55X_SET_LED0_ON; // Turn on LED0 (Green AUX LED on Sleipnir)
    } else {
      C55X_SET_LED1_ON; // Turn on LED1 (Red AUX LED on Sleipnir)
    }

    OtxDspCommandCompleteRc(hHandle, dspRc);
    
  } else if (nCode == OTXDSP_SDK_IO_INIT_TIMER) {
    unsigned short dspRc;
    dspRc = OtxDspInitTimer();
    OtxDspCommandCompleteRc(hHandle, dspRc);
    
  // Application specific IO Control Codes
  } else if (nCode == DEMO1_IOCTL_SET_BLINK_FREQ) {
    Demo1IoInSetBlinkFreqS *pInS = (Demo1IoInSetBlinkFreqS *)(OTXDSP_SDK_IOCTL_DATA_ADDR/2);

    g_blinkCountMax = pInS->nBlinkCountFactor * 500L;

    // Save these values in the debug struct. To be retrieved when the DEMO1_IOCTL_GET_DEBUG_DATA control is issued
    g_debugDataS.nData0     = pInS->nBlinkCountFactor;
    g_debugDataS.nData1     = pInS->nDummyData;
    g_debugDataS.nData2     = nDataLen;
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_S_OK);

  } else if (nCode == DEMO1_IOCTL_SET_CO_DEBUG_NOTIFICATIONS) {
    Demo1IoInSetCoDebugNotificationsS *pInS = (Demo1IoInSetCoDebugNotificationsS *)(OTXDSP_SDK_IOCTL_DATA_ADDR/2);

    g_coDebugDataNotifications = pInS->coDebugDataNotifications;
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_S_OK);

  } else if (nCode == DEMO1_IOCTL_GET_BLINK_FREQ) {
    Demo1IoOutGetBlinkFreqS sOut;

    sOut.nResult = OTXDSP_SDK_S_OK;

    sOut.nBlinkCountMaxLow  = (unsigned short) g_blinkCountMax;
    sOut.nBlinkCountMaxHigh = (unsigned short)(g_blinkCountMax >> 16);
    sOut.nBlinkCountLow     = (unsigned short) g_blinkCount;
    sOut.nBlinkCountHigh    = (unsigned short)(g_blinkCount >> 16);

#if 1
    sOut.nBlinkCountMaxLow  = (unsigned short) 5678;
    sOut.nBlinkCountMaxHigh = (unsigned short) 0;
    sOut.nBlinkCountLow     = (unsigned short) 12;
    sOut.nBlinkCountHigh    = (unsigned short) 0;
#endif


#if 0
    // Add a delay (to test blocking calls from the host application)
    {
      long j, k;
      for (j = 0; j < 200000; j++) {
        k = j%10;
      }
    }
#endif

    OtxDspCommandCompleteData(hHandle, (unsigned short *)&sOut, sizeof(Demo1IoOutGetBlinkFreqS));

  } else if (nCode == DEMO1_IOCTL_GET_HPI_EVENT_BUF_DIAG) {
    Demo1IoOutGetHpiEventBufDiagS sOut;

    sOut.nResult = OTXDSP_SDK_S_OK;

    sOut.cwFreeBuffer    = g_hpiEventBufDiag.cwFreeBuffer;
    sOut.cwMinFreeBuffer = g_hpiEventBufDiag.cwMinFreeBuffer;  
    sOut.coLostEvents    = g_hpiEventBufDiag.coLostEvents;     

    OtxDspCommandCompleteData(hHandle, (unsigned short *)&sOut, sizeof(Demo1IoOutGetHpiEventBufDiagS));

  } else if (nCode == DEMO1_IOCTL_GET_DEBUG_DATA) {

    // This is an example of a non-blocking call
    // We must confirm the IoControl command before we can send any other events
    // back to the host.
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_S_OK);

    // A non-blocking call would normally take some time to execute.
    // Normally the application should save the Task Reference value, perform the task,
    // then call OtxDspSetHostEvent using the save Task Reference value.
    // However, in this example we call OtxDspSetHostEvent immediately.

    //g_debugDataS.nData0 = g_intmVal;
    //g_debugDataS.nData1 = 1;
    //g_debugDataS.nData2 = 2;
    {
      short i;
      for (i = 0; i < 299; i++) {
        g_debugDataS.sqBuf[i] = i;
      }
      g_debugDataS.sqBuf[299]++;
    }

    { 
      short i;
      // For debugging of the HPI Event Data buffer we here allow a number of 
      // DEMO1_NOTIFY_DEBUG_DATA events to be entered into the buffer at once.
      // The g_coDebugDataNotifications variable is set by the DEMO1_IOCTL_SET_CO_DEBUG_NOTIFICATIONS code.
      for (i = 0; i < g_coDebugDataNotifications; i++) {
        OtxDspSetHostEvent(DEMO1_NOTIFY_DEBUG_DATA, hHandle, nTaskRef, (unsigned short *)&g_debugDataS, sizeof(Demo1NotifyGetDebugDataS));
      }
    }

  } else {
    OtxDspCommandCompleteRc(hHandle, OTXDSP_SDK_E_INVALIDARG);
  }

}


/*--------------------------------------------------------------------------*\
  @func

    Call-back routine call when we have receive interrupt from the
    serial port indicating the the receive buffer is either full or
    half-full.

    This function is called from the processing task.
    TO BE RE-IMPLEMENTED BY THE USER.

\*--------------------------------------------------------------------------*/
void OtxDspServiceRxSerialPort(
  void
)
{
  /* Add your code here .... */
}


/*--------------------------------------------------------------------------*\
  @func

    Call-back routine call when we have transmit interrupt from the
    Buffered Serial Port indicating the the transmit buffer is either empty or
    half-empty.

    This function is called from the processing task.
    TO BE RE-IMPLEMENTED BY THE USER.

\*--------------------------------------------------------------------------*/
void OtxDspServiceTxSerialPort(
  void
)
{
  /* Add your code here .... */
}


